#!/usr/bin/env python

import xml.etree.ElementTree as ET
import exceptions
import struct
import os
import json
import re
import sys

def num(s):
    try:
        return int(s)
    except exceptions.ValueError:
        return float(s)

class Tmx2Json:
    def __init__(self):
        self.currentDir = ''
        self.numerical_keys = [
            'width', 'height', 'tilewidth', 'tileheight',
            'x', 'y', 'gid', 'firstgid', 'opacity',
            'imageheight', 'imagewidth', 'margin',
            'spacing', 'version'
        ]
        self.special_map = {
            '.layer.data': self.get_layer_data,
            '.properties': self.get_properties,
            '.objectgroup': self.get_objectgroup,
            '.tileset': self.get_tileset,
            '.map': self.get_map
        }

    def get_attrib(self, attribs, key):
        if key in self.numerical_keys:
            return num(attribs[key])
        return attribs[key]

    def set_attribs(self, table, node):
        for key in node.attrib:
            table[key] = self.get_attrib(node.attrib, key)
        return table

    def get_layer_data(self, data, upper_tag_stack):
        result = []
        d = data.text.strip()
        if 'encoding' in data.attrib:
            if data.attrib['encoding'] == 'csv':
                result = d = [int(s) for s in data.text.split(',')]
            elif data.attrib['encoding'] == 'base64':
                d = d.decode(data.attrib['encoding'])
                if 'compression' in data.attrib:
                    d = d.decode(data.attrib['compression'])
                result = list(struct.unpack('<' + str(len(d)/4) + 'I', d))
        return result

    def get_properties(self, properties, upper_tag_stack):
        result = {}
        for child in properties:
            if child.tag == 'property':
                result[child.attrib['name']] = child.attrib['value']
        return result

    def get_objectgroup(self, node, upper_tag_stack):
        tag_stack = upper_tag_stack + '.' + node.tag
        result = {}
        self.set_attribs(result, node)

        result['objects'] = []
        for child in node:
            result['objects'].append(self.get_node_as_object(child, tag_stack))
            #HACK to give all objects a properties attrib
            if 'properties' not in result['objects'][-1]:
                result['objects'][-1]['properties'] = {}

        return result

    def get_tileset(self, node, upper_tag_stack):
        tag_stack = upper_tag_stack + '.' + node.tag
        result = {}

        tileset_node = node
        if 'firstgid' in tileset_node.attrib:
            result['firstgid'] = tileset_node.attrib['firstgid']

        if 'source' in tileset_node.attrib:
            tileset_node = ET.parse(self.currentDir + node.attrib['source']).getroot()
            if tileset_node.tag != 'tileset':
                return result

        self.set_attribs(result, tileset_node)

        properties = tileset_node.find('properties')
        if properties != None:
            result['properties'] = self.get_node_as_object(properties, tag_stack)

        image = tileset_node.find('image')
        if image != None:
            result['image'] = image.attrib['source']
            result['imageheight'] = image.attrib['height']
            result['imagewidth'] = image.attrib['width']

        tiles = tileset_node.findall('tile')
        if tiles != []:
            result['tileproperties'] = tileproperties = {}
            for tile in tiles:
                tileproperties[tile.attrib['id']] = tile_json = {}
                for property in tile.findall('properties/property'):
                    tile_json[property.attrib['name']] = property.attrib['value']

        return result

    def get_map(self, node, upper_tag_stack):
        tag_stack = upper_tag_stack + '.' + node.tag
        json_result = {}

        self.set_attribs(json_result, node)
        json_result['layers'] = layers = []
        json_result['tilesets'] = tilesets = []

        for child in node:
            if child.tag == 'layer':
                layer = self.get_node_as_object(child, tag_stack)
                layer['type'] = 'tilelayer'
                layers.append(layer)
            elif child.tag == 'objectgroup':
                layer = self.get_node_as_object(child, tag_stack)
                layer['type'] = 'objectgroup'
                layers.append(layer)
            elif child.tag == 'tileset':
                tileset = self.get_node_as_object(child, tag_stack)
                tilesets.append(tileset)

        return json_result

    def get_node_as_object(self, node, upper_tag_stack=''):
        tag_stack = upper_tag_stack + '.' + node.tag
        for key in self.special_map:
            if tag_stack.endswith(key):
                return self.special_map[key](node, tag_stack)

        result = {}
        self.set_attribs(result, node)
        for child in node:
            result[child.tag] = self.get_node_as_object(child, tag_stack)
        return result


    def tmx2json(self, filename):
        self.currentDir = os.path.dirname( os.path.realpath( filename ) ) + '/'

        root = ET.parse(filename).getroot()

        json_result = self.get_node_as_object(root)
        json_str = json.dumps(json_result, sort_keys=True, indent=2, separators=(',',':'))
        json_str = re.sub(r'\s\s+(\d+)', lambda match: r' {}'.format(match.group(1), json_str), json_str) # Make the data array look nicer

        outfilename = filename[:(filename.rfind('.') if '.' in filename else None)] + '.json'
        outfile = open(outfilename, 'wb')
        outfile.write(json_str)
        outfile.close()


if __name__ == '__main__':
    if len(sys.argv) < 2:
        sys.stderr.write("%s requires a file argument.\n" % sys.argv[0])
        sys.exit(1)

    filename = sys.argv[1]
    exporter = Tmx2Json()
    exporter.tmx2json(filename)

